package meerkat;

import meerkat.protobuf.VoterRegistry.*;

/**
 * Created by Vladimir Eliezer Tokarev on 1/22/2016.
 * provides voters management options
 */
public interface VoterRegistry {

    /**
     * Created by Vladimir Eliezer Tokarev on 1/22/2016.
     * This interface will handle the end of methods of RegistryInstance
     * in case of exception the handleFailure will be called
     */
    interface RegistryCallBack<T> {
        void handleResult(T result);
        void handleFailure(Throwable throwable);
    }

    /**
     * Adds new voter to the bulletin-board
     *      Passes true to callBack.handleResult  if the actions succeeded else false
     *
     * @param voterInfo    protobuff object that represents voter information
     * @param callBack     when the adding voter done callBack.handleResult will be called
     * @return void
     */
    void addVoter(VoterInfo voterInfo, RegistryCallBack callBack);

    /**
     * Adding given voter to given group
     *        Passes the group  to callBack.handleResult  if the actions succeeded else null
     *
     * @param voterGroup protobuff object that is coupling of voterId to groupId
     * @param callBack when the adding voter done callBack.handleResult will be called
     * @return true if the adding action succeeded else return false
     */
    void addToGroups(VoterRegistryMessage voterGroup, RegistryCallBack callBack);

    /**
     * Sets that the voter have voted
     *          Passes true to callBack.handleResult  if the actions succeeded else false
     *
     * @param voterId       id tag string
     * @param callBack when the adding voter done callBack.handleResult will be called
     * @return true if the set voted succeed else false
     */
    void setVoted(VoterID voterId, RegistryCallBack callBack);

    /**
     * Requests all the groups that the given id voter is in
     *         Passes wanted groups to callback.handleResult if the actions succeeded else null
     *
     * @param groupID       id tag string
     * @param callBack when the adding voter done callBack.handleResult will be called
     */
    void getGroups(GroupID groupID, RegistryCallBack callBack);

    /**
     * Retrieves list of strings that represents voter
     *          Passes wanted data to callback.handleResult if the actions succeeded else null
     *
     * @param voterID       id tag string
     * @param callBack when the adding voter done callBack.handleResult will be called
     * @return list of strings (empty list if the lookup failed)
     */
    void getPersonIDDetails(VoterID voterID, RegistryCallBack callBack);
}

